from __future__ import absolute_import
import collections

class FrozenDict(collections.Mapping):
    '''Immutable dictionary.'''

    def __init__(self, *args, **kwargs):
        self._dict = dict(*args, **kwargs)
        self._hash = None

    def __getitem__(self, key):
        '''Pass-through to internal dict.'''
        return self._dict[key]

    def __iter__(self):
        '''Pass-through to internal dict.'''
        return iter(self._dict)

    def __len__(self):
        '''Pass-through to internal dict.'''
        return len(self._dict)

    def __hash__(self):
        '''Caches lazily-computed hash value.'''
        if self._hash is None:
            self._hash = hash(frozenset(self._dict.items()))
        return self._hash

    def __repr__(self):
        items = ('{}: {}'.format(repr(k),repr(v)) for k,v in sorted(self.items()))
        return '{}({{{}}})'.format(type(self).__name__, ', '.join(items))




from setuptools import setup

setup(
    name='icicle',
    version='1.0.0',
    description='Python FrozenDict',
    url='https://github.com/pcattori/icicle',
    author='Pedro Cattori',
    author_email='pcattori@gmail.com',
    license='MIT',
    packages=['icicle'],
    test_suite='tests',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5'
    ]
)
